<?php /* -*- PHP -*- */

define ("_UNIT_TEST_", true);

if (!defined ("HGREPOS"))
    define ("HGREPOS", "tmp/hg");
if (!defined ("LOGFILE"))
    define ("LOGFILE", "tmp/logging.log");
if (!defined ("LOGLEVEL"))
    define ("LOGLEVEL", 3);

function init_hgrepos () {

    if (file_exists (HGREPOS)) {
        passthru ("chmod -R a+w " . HGREPOS);
        passthru ("rm -rf " . HGREPOS);
    }

    passthru ("hg init " . HGREPOS . "/a");
    passthru ("hg init " . HGREPOS . "/b");
    passthru ("mkdir   " . HGREPOS . "/b/a");
    passthru ("hg init " . HGREPOS . "/c");
    passthru ("hg init " . HGREPOS . "/c/a");
    passthru ("mkdir   " . HGREPOS . "/c/b");
    passthru ("hg init " . HGREPOS . "/d/a");
    passthru ("hg init " . HGREPOS . "/d/a/a");
    passthru ("hg init " . HGREPOS . "/d/b");

    if ($fd = fopen (HGREPOS . "/a/.hg/hgrc", "w")) {
        $len = fwrite ($fd, "[web]\ncontact = Laurent Mazet\ndescription = Mercurial Headquarters\nstyle = gitweb");
        fclose ($fd);
    }

    if ($fd = fopen (".htaccess", "w")) {
        $len = fwrite ($fd, "Order allow,deny\nAllow from all\n\nAuthType Basic\n\nAuthName \"Mercurial HeadQuarter\"\nAuthBasicProvider file\nAuthUserFile " . HGREPOS . "/.htpassword\n#AuthLDAPURL \"ldap://ldap.my_company.my_ext/ou=People,dc=my_company,dc=my_ext?uid?sub\"\nRequire valid-user\nrequire user admin\n");
        fclose ($fd);
    }


    if ($fd = fopen (HGREPOS . "/.htpassword", "w")) {
        $len = fwrite ($fd, "admin:xGTJeDo9pPVvM\nfoo:L041BVWz6Z9Ws\nbar:keILe1xPFD4GA\n");
        fclose ($fd);
    }

    if ($fd = fopen (HGREPOS . "/.htaccess", "w")) {
        $len = fwrite ($fd, "Order allow,deny\nAllow from all\n\nAuthType Basic\n\nAuthName \"Mercurial HeadQuarter\"\nAuthBasicProvider file\nAuthUserFile " . HGREPOS . "/.htpassword\n#AuthLDAPURL \"ldap://ldap.my_company.my_ext/ou=People,dc=my_company,dc=my_ext?uid?sub\"\nRequire valid-user\n");
        fclose ($fd);
    }

}

$xmlfile = "tmp/output.xml";
if (file_exists ($xmlfile))
    passthru ("rm -rf $xmlfile");

$access = ".htaccess";
if (file_exists ($access))
    passthru ("rm -rf $access");

function ob_file_callback ($buffer) {
    global $xmlfile;
    if ($fd = fopen ($xmlfile, "a")) {
        $nb = fwrite ($fd, $buffer, strlen ($buffer));
        fclose ($fd);
    }
}

function count_files ($name) {
    if (!is_dir ($name))
        return 1;
    $dh = opendir ($name);
    if (!$dh)
        return 0;
    $count = 0;
    while (($f = readdir ($dh)) != false) {
        if ((strcmp ($f, ".") == 0) || (strcmp ($f, "..") == 0)) continue;
        $count += count_files ($f);
    }
    return $count;
}

function compare_files ($name, &$tree, &$new, &$modify) {

    if (!is_dir ($name)) {
        $md5 = @md5_file ($name);
        if (array_key_exists ($name, $tree)) {
            if (strcmp ($md5, $tree[$name]) != 0) {
                $modify[$name] = $md5;
            }
        } else {
            $new[$name] = $md5;
        }
        $tree[$name] = $md5;
        return;
    }

    if (!($dh = opendir ($name)))
        return;

    while (($f = readdir ($dh)) != false) {
        if ((strcmp ($f, ".") == 0) || (strcmp ($f, "..") == 0)) continue;
        compare_files ("$name/$f", $tree, $new, $modify);
    }
}

function do_unit_test ($xmloutput, $new_expected = null, $modify_expected = null, $delete_expected = null) {
    global $xmlfile;

    $hgrepos = array ();
    $new = array ();
    $modify = array ();
    compare_files (HGREPOS, $hgrepos, $new, $modify);

    if (count ($modify_expected) != 0) {
      foreach ($modify_expected as $key => $md5)
        if ((file_exists ($key)) && (!array_key_exists ($key, $hgrepos)))
          $hgrepos[$key] = md5_file ($key);
    }
    ob_start ("ob_file_callback");

    require ("../func.php");

    ob_end_flush ();

    $data = (file_exists ($xmlfile)) ? file_get_contents ($xmlfile) : "";
    if (strlen ($xmloutput) != 0) {
        echo "check xml output\n";
        if (strcmp ($data, $xmloutput) != 0) exit (1);
    }

    $new = array ();
    $modify = array ();
    compare_files (HGREPOS, $hgrepos, $new, $modify);

    echo "check for new files\n";
    foreach ($new as $key => $md5) {
        if ((count ($new_expected) != 0) && 
            (array_key_exists ($key, $new_expected))) {
            if (strcmp ($new_expected[$key], $md5) == 0) {
                unset ($new_expected[$key]);
                unset ($new[$key]);
            } else {
                    echo "incorrect md5sum for new file '$key'\n";
                    exit (1);
            }
        } else {
            echo "unexpected new file '$key'\n";
            exit (1);
        }
    }
    if (count ($new_expected) != 0) {
      foreach ($new_expected as $key => $md5)
        if ((file_exists ($key)) && (strcmp (md5_file ($key), $md5) == 0))
          unset ($new_expected[$key]);
    }
    if (count ($new_expected) != 0) {
        $names = array_reduce (array_keys ($new_expected), function ($x, $y) { return is_null ($x) ? "'$y'" : sprintf ("%s, '%s'", $x, $y); });
 "";
        echo "missing expected new file(s): $names\n";
        exit (1);
    }

    echo "check for modify files\n";
    foreach ($modify as $key => $md5) {
        if ((count ($modify_expected) != 0) && 
            (array_key_exists ($key, $modify_expected))) {
            if (strcmp ($modify_expected[$key], $md5) == 0) {
                unset ($modify_expected[$key]);
                unset ($modify[$key]);
            } else {
                echo "incorrect md5sum for modify file '$key'\n";
                exit (1);
                }
        } else {
            echo "unexpected modify file '$key'\n";
            exit (1);
        }
    }
    if (count ($modify_expected) != 0) {
      foreach ($modify_expected as $key => $md5)
        if ((file_exists ($key)) && (array_key_exists ($key, $hgrepos)) &&
            (strcmp ($hgrepos[$key], $md5) != 0) && (strcmp (md5_file ($key), $md5) == 0))
          unset ($modify_expected[$key]);
    }
    if (count ($modify_expected) != 0) {
        $names = array_reduce (array_keys ($modify_expected), function ($x, $y) { return is_null ($x) ? "'$y'" : sprintf ("%s, '%s'", $x, $y); });
        echo "missing expected modify file(s): $names\n";
        exit (1);
    }

    echo "check for deleted files\n";
    foreach ($hgrepos as $key => $md5) {
        echo "file: '$key'\n";
        if (!file_exists ($key)) {
            if ((count ($delete_expected) == 0) || 
                (!array_key_exists ($key, $delete_expected))) {
                echo "unexpected delete file '$key'\n";
                exit (1);
            }
            unset ($delete_expected["$key"]);
        }
    }
    if (count ($delete_expected) != 0) {
      foreach ($delete_expected as $key => $md5)
        if (!file_exists ($key))
          unset ($delete_expected[$key]);
    }
    if (count ($delete_expected) != 0) {
        $names = array_reduce (array_keys ($delete_expected), function ($x, $y) { return is_null ($x) ? "'$y'" : sprintf ("%s, '%s'", $x, $y); });
 "";
        echo "missing expected delete file(s): $names\n";
        exit (1);
    }

    return $data;
}

function import_xml ($data) {

    /* Create XML parser */
    $xml_parser = xml_parser_create ();
    xml_parser_set_option ($xml_parser, XML_OPTION_CASE_FOLDING, 0);
    xml_parser_set_option ($xml_parser, XML_OPTION_SKIP_WHITE, 1);
    xml_parse_into_struct ($xml_parser, $data, $vals, $index);
    xml_parser_free ($xml_parser);

    $params = array();
    $level = array();
    foreach ($vals as $xml_elem) {
        /* Create key from atribute */
        if ($xml_elem['type'] == 'open') {
            if (array_key_exists('attributes',$xml_elem)) {
                list($level[$xml_elem['level']],$extra) =
                    array_values($xml_elem['attributes']);
            } else {
                $level[$xml_elem['level']] = $xml_elem['tag'];
            }
        }

        /* Go back to previous level */
        if ($xml_elem['type'] == 'complete') {
            $php_stmt = '$params';
            $pt = $params;
            for ($start_level = 1; $start_level < $xml_elem['level']; $start_level++) {
                $php_stmt .= '[$level['.$start_level.']]';
                if (!array_key_exists ($level[$start_level], $pt))
                    eval ($php_stmt . ' = array();');
                eval ('$pt  = ' . $php_stmt . ';');
            }
            $php_stmt .= '[$xml_elem[\'tag\']]';
            if (!array_key_exists ($xml_elem['tag'], $pt))
                eval ($php_stmt . ' = array ();');
            $value = (array_key_exists ("value", $xml_elem)) ? $xml_elem["value"] : null;
            eval('array_push (' . $php_stmt . ', $value);');
        }
    }

    return $params;
}

function _file_write ($name, $string) {

  $fd = @fopen ($name, "w");
  if ($fd) {
    @fwrite ($fd, $string, strlen ($string));
    @fclose ($fd);
  }
}

?>
